/**
 * This file is part of Haketilo.
 *
 * Function: Broadcast messages to different execution contexts of the extension
 *
 * Copyright (C) 2021 Wojtek Kosior
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * As additional permission under GNU GPL version 3 section 7, you
 * may distribute forms of that code without the copy of the GNU
 * GPL normally required by section 4, provided you include this
 * license notice and, in case of non-source distribution, a URL
 * through which recipients can access the Corresponding Source.
 * If you modify file(s) with this exception, you may extend this
 * exception to your version of the file(s), but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * As a special exception to the GPL, any HTML file which merely
 * makes function calls to this code, and for that purpose
 * includes it by reference shall be deemed a separate work for
 * copyright law purposes. If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * I, Wojtek Kosior, thereby promise not to sue for violation of this file's
 * license. Although I request that you do not make use of this code in a
 * proprietary program, I am not going to enforce this in court.
 */

#FROM common/message_server.js IMPORT connect_to_background

function sender_connection()
{
    return {
	type: "sender",
	port: connect_to_background("broadcast_send")
    };
}
#EXPORT sender_connection

function out(sender_conn, channel_name, value)
{
    sender_conn.port.postMessage(["broadcast", channel_name, value]);
}
#EXPORT out

/*
 * prepare()'d message will be broadcasted if the connection is closed or when
 * flush() is called. All messages prepared by given sender will be discarded
 * when discard() is called.
 *
 * Timeout will cause the prepared message to be broadcasted after given number
 * of miliseconds unless discard() is called in the meantime. It is mostly
 * useful as a fallback in case this connection end gets nuked (e.g. because
 * browser window is closed) and onDisconnect event is not dispatched to
 * background script's Port object. This is only an issue for browsers as old as
 * IceCat 60: https://bugzilla.mozilla.org/show_bug.cgi?id=1392067
 *
 * Timeout won't be scheduled at all if its value is given as 0.
 */
function prepare(sender_conn, channel_name, value, timeout=5000)
{
    sender_conn.port.postMessage(["prepare", channel_name, value, timeout]);
}
#EXPORT prepare

function discard(sender_conn)
{
    sender_conn.port.postMessage(["discard"]);
}
#EXPORT discard

function flush(sender_conn)
{
    sender_conn.port.postMessage(["flush"]);
}
#EXPORT flush

function listener_connection(cb)
{
    const conn = {
	type: "listener",
	port: connect_to_background("broadcast_listen")
    };

    conn.port.onMessage.addListener(cb);

    return conn;
}
#EXPORT listener_connection

function subscribe(listener_conn, channel_name)
{
    listener_conn.port.postMessage(["subscribe", channel_name]);
}
#EXPORT subscribe

function unsubscribe(listener_conn, channel_name)
{
    listener_conn.port.postMessage(["unsubscribe", channel_name]);
}
#EXPORT unsubscribe

function close(conn)
{
    if (conn.type === "sender")
	flush(conn);
    setTimeout(() => conn.port.disconnect());
}
#EXPORT close
