/**
 * This file is part of Haketilo.
 *
 * Function: Making requests to remote repositories through a response cache in
 *           operating in the content script of a browser tab.
 *
 * Copyright (C) 2022 Wojtek Kosior <koszko@koszko.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * As additional permission under GNU GPL version 3 section 7, you
 * may distribute forms of that code without the copy of the GNU
 * GPL normally required by section 4, provided you include this
 * license notice and, in case of non-source distribution, a URL
 * through which recipients can access the Corresponding Source.
 * If you modify file(s) with this exception, you may extend this
 * exception to your version of the file(s), but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * As a special exception to the GPL, any HTML file which merely
 * makes function calls to this code, and for that purpose
 * includes it by reference shall be deemed a separate work for
 * copyright law purposes. If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * I, Wojtek Kosior, thereby promise not to sue for violation of this file's
 * license. Although I request that you do not make use of this code in a
 * proprietary program, I am not going to enforce this in court.
 */

#FROM common/browser.js IMPORT browser

/*
 * This is a complementary function to error_data_jsonifiable() in
 * common/misc.js.
 */
function error_from_jsonifiable_data(jsonifiable) {
    const arg_props = ["message", "fileName", "lineNumber"];
    return new window[jsonifiable.name](...arg_props.map(p => jsonifiable[p]));
}

/*
 * Make it possible to recover a Response object. This is a complementary
 * function to response_data_jsonifiable() in background/CORS_bypass_server.js.
 */
function response_from_jsonifiable_data(jsonifiable) {
    const body = jsonifiable.body, body_len = body.length / 2;
    const body_buf = new Uint8Array(body_len);

    for (let i = 0; i < body_len; i++)
	body_buf[i] = parseInt(`0x${body.substring(i * 2, i * 2 + 2)}`);

    const init = {
	status:     jsonifiable.status,
	statusText: jsonifiable.statusText,
	headers:    new Headers(jsonifiable.headers)
    };

    return new Response(body_buf, init);
}

async function indirect_fetch(tab_id, url) {
    const msg = ["repo_query", url];
    const jsonifiable = await browser.tabs.sendMessage(tab_id, msg);

    if ("error" in jsonifiable)
	throw error_from_jsonifiable_data(jsonifiable);

    return response_from_jsonifiable_data(jsonifiable);
}
#EXPORT indirect_fetch
